@@ -29,12 +29,14 @@
 #include "mos_os.h"
 #include "mos_interface.h"
 #include "mhw_vebox.h"
-#include "renderhal.h"
+#include "renderhal_legacy.h"
 #include "vphal_renderer.h"
 #include "mos_solo_generic.h"
 #include "media_interfaces_vphal.h"
 #include "media_interfaces_mhw.h"
 #include "mhw_vebox_itf.h"
+#include "mos_oca_rtlog_mgr.h"
+#include "vp_user_setting.h"
 
 //!
 //! \brief    Allocate VPHAL Resources
@@ -50,11 +52,14 @@ MOS_STATUS VphalState::Allocate(
     const VphalSettings       *pVpHalSettings)
 {
     MHW_VEBOX_GPUNODE_LIMIT     GpuNodeLimit;
-    RENDERHAL_SETTINGS          RenderHalSettings;
+    RENDERHAL_SETTINGS_LEGACY   RenderHalSettings;
     MOS_GPU_NODE                VeboxGpuNode;
     MOS_GPU_CONTEXT             VeboxGpuContext;
+    bool                        checkGpuCtxOverwriten = false;
+    bool                        addGpuCtxToCheckList  = false;
     MOS_STATUS                  eStatus;
 
+    VPHAL_PUBLIC_CHK_NULL(m_osInterface);
     VPHAL_PUBLIC_CHK_NULL(pVpHalSettings);
     VPHAL_PUBLIC_CHK_NULL(m_renderHal);
 
@@ -90,9 +95,23 @@ MOS_STATUS VphalState::Allocate(
         }
     }
 
+    if (IsApoEnabled() &&
+        m_osInterface->apoMosEnabled &&
+        m_osInterface->bSetHandleInvalid)
+    {
+        checkGpuCtxOverwriten = true;
+    }
+    addGpuCtxToCheckList = checkGpuCtxOverwriten && !IsGpuContextReused(m_renderGpuContext);
+
+    if (IsRenderContextBasedSchedulingNeeded())
+    {
+        Mos_SetVirtualEngineSupported(m_osInterface, true);
+        m_osInterface->pfnVirtualEngineSupported(m_osInterface, true, true);
+    }
+
     // Create Render GPU Context
     {
-        MOS_GPUCTX_CREATOPTIONS createOption;
+        MOS_GPUCTX_CREATOPTIONS_ENHANCED createOption = {};
         VPHAL_PUBLIC_CHK_STATUS(m_osInterface->pfnCreateGpuContext(
             m_osInterface,
             m_renderGpuContext,
@@ -109,9 +128,8 @@ MOS_STATUS VphalState::Allocate(
     // In legacy path, stream 0 gpu contexts could be reused, and will keep these stream 0 gpu contexts alive
     // But its mapping relation could be overwritten in APO path
     // Now, don't reuse these stream 0 gpu contexts overwritten by MediaContext in APO path 
-    if (IsApoEnabled() && 
-        m_osInterface->apoMosEnabled && 
-        m_osInterface->bSetHandleInvalid)
+    // Can not add reused GPU context
+    if (addGpuCtxToCheckList)
     {
         AddGpuContextToCheckList(m_renderGpuContext);
     }
@@ -145,6 +163,8 @@ MOS_STATUS VphalState::Allocate(
         VeboxGpuNode    = (MOS_GPU_NODE)(GpuNodeLimit.dwGpuNodeToUse);
         VeboxGpuContext = (VeboxGpuNode == MOS_GPU_NODE_VE) ? MOS_GPU_CONTEXT_VEBOX : MOS_GPU_CONTEXT_VEBOX2;
 
+        addGpuCtxToCheckList = checkGpuCtxOverwriten && !IsGpuContextReused(VeboxGpuContext);
+      
         // Create VEBOX/VEBOX2 Context
         VPHAL_PUBLIC_CHK_STATUS(m_veboxInterface->CreateGpuContext(
             m_osInterface,
@@ -152,9 +172,7 @@ MOS_STATUS VphalState::Allocate(
             VeboxGpuNode));
 
         // Add gpu context entry, including stream 0 gpu contexts
-        if (IsApoEnabled() && 
-            m_osInterface->apoMosEnabled && 
-            m_osInterface->bSetHandleInvalid)
+        if (addGpuCtxToCheckList)
         {
             AddGpuContextToCheckList(VeboxGpuContext);
         }
@@ -659,17 +677,13 @@ VphalFeatureReport* VphalState::GetRenderFeatureReport()
 //!           - Caller must call pfnAllocate to allocate all VPHAL states and objects.
 //! \param    [in] pOsInterface
 //!           OS interface, if provided externally - may be nullptr
-//! \param    [in] pOsDriverContext
-//!           OS driver context (UMD context, pShared, ...)
 //! \param    [in,out] peStatus
 //!           Pointer to the MOS_STATUS flag.
 //!           Will assign this flag to MOS_STATUS_SUCCESS if successful, otherwise failed
 //!
 VphalState::VphalState(
         PMOS_INTERFACE          pOsInterface,
-        PMOS_CONTEXT            pOsDriverContext,
         MOS_STATUS              *peStatus) :
-        m_gpuAppTaskEvent(nullptr),
         m_platform(),
         m_skuTable(nullptr),
         m_waTable(nullptr),
@@ -693,9 +707,12 @@ VphalState::VphalState(
     m_skuTable = m_osInterface->pfnGetSkuTable(m_osInterface);
     m_waTable  = m_osInterface->pfnGetWaTable (m_osInterface);
 
-    m_renderHal = (PRENDERHAL_INTERFACE)MOS_AllocAndZeroMemory(sizeof(*m_renderHal));
+    m_userSettingPtr = m_osInterface->pfnGetUserSettingInstance(m_osInterface);
+    VpUserSetting::InitVpUserSetting(m_userSettingPtr);
+
+    m_renderHal = (PRENDERHAL_INTERFACE_LEGACY)MOS_AllocAndZeroMemory(sizeof(RENDERHAL_INTERFACE_LEGACY));
     VPHAL_PUBLIC_CHK_NULL(m_renderHal);
-    VPHAL_PUBLIC_CHK_STATUS(RenderHal_InitInterface(
+    VPHAL_PUBLIC_CHK_STATUS(RenderHal_InitInterface_Legacy(
         m_renderHal,
         &m_cpInterface,
         m_osInterface));
@@ -716,6 +733,13 @@ VphalState::~VphalState()
 {
     MOS_STATUS              eStatus;
 
+    // Wait for all cmd before destroy gpu resource
+    if (m_osInterface && m_osInterface->pfnWaitAllCmdCompletion && m_osInterface->bDeallocateOnExit)
+    {
+        VP_PUBLIC_NORMALMESSAGE("WaitAllCmdCompletion in VphalState::~VphalState");
+        m_osInterface->pfnWaitAllCmdCompletion(m_osInterface);
+    }
+
     // Destroy rendering objects (intermediate surfaces, BBs, etc)
     if (m_renderer)
     {
@@ -792,7 +816,13 @@ VphalState* VphalState::VphalStateFactory(
     PMOS_CONTEXT            pOsDriverContext,
     MOS_STATUS              *peStatus)
 {
-    return VphalDevice::CreateFactory(pOsInterface, pOsDriverContext, peStatus);
+    VphalState* vphalState = dynamic_cast<VphalState*>(VphalDevice::CreateFactory(pOsInterface, pOsDriverContext, peStatus));
+    if(nullptr == vphalState)
+    {
+        VPHAL_PUBLIC_ASSERTMESSAGE("vphal state pointer is nullptr");
+        *peStatus = MOS_STATUS_NULL_POINTER;
+    }
+    return vphalState;
 }
 
 //!
@@ -858,11 +888,7 @@ MOS_STATUS VphalState::GetStatusReport(
             continue;
         }
 
-#if (LINUX || ANDROID)
-        dwGpuTag    = pOsContext->GetGPUTag(m_osInterface, pStatusEntry->GpuContextOrdinal);
-#else
-        dwGpuTag    = m_osInterface->pfnGetGpuStatusSyncTag(m_osInterface, pStatusEntry->GpuContextOrdinal);
-#endif
+        dwGpuTag           = m_osInterface->pfnGetGpuStatusSyncTag(m_osInterface, pStatusEntry->GpuContextOrdinal);
         bDoneByGpu         = (dwGpuTag >= pStatusEntry->dwTag);
         bFailedOnSubmitCmd = (pStatusEntry->dwStatus == VPREP_ERROR);
 
@@ -1044,4 +1070,37 @@ MOS_STATUS VphalState::DestroyGpuContextWithInvalidHandle()
 #endif
 
     return MOS_STATUS_SUCCESS;
-}
\ No newline at end of file
+}
+
+//!
+//! \brief    Check whether GPU context is reused or not
+//! \details  Check whether GPU context is reused or not
+//! \param    MOS_GPU_CONTEXT mosGpuConext
+//!           [in] Mos GPU context
+//! \return   bool
+//!           Return true if is reused, otherwise false
+//!
+bool VphalState::IsGpuContextReused(
+    MOS_GPU_CONTEXT mosGpuContext)
+{
+    bool reuseContextFlag = false;
+#if !EMUL
+    MOS_GPU_CONTEXT originalGpuCtxOrdinal = m_osInterface->CurrentGpuContextOrdinal;
+    if (MOS_SUCCEEDED(m_osInterface->pfnSetGpuContext(m_osInterface, mosGpuContext)))
+    {
+        VPHAL_PUBLIC_NORMALMESSAGE("Reuse mos GPU context %d. GPU handle %d", (uint32_t)mosGpuContext, m_osInterface->CurrentGpuContextHandle);
+        reuseContextFlag = true;
+    }
+
+    if (originalGpuCtxOrdinal < MOS_GPU_CONTEXT_MAX &&
+        m_osInterface->CurrentGpuContextOrdinal != originalGpuCtxOrdinal)
+    {
+        //Recover original settings
+        if (m_osInterface->pfnSetGpuContext(m_osInterface, originalGpuCtxOrdinal) != MOS_STATUS_SUCCESS)
+        {
+            VPHAL_PUBLIC_NORMALMESSAGE("Have not recovered original GPU Context settings in m_osInterface");
+        }
+    }
+#endif
+    return reuseContextFlag;
+}
