@@ -138,7 +138,8 @@ static struct LinuxCodecInfo PvcCodecInfo =
 static bool InitTglMediaSkuExt(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
                              struct LinuxDriverInfo *drvInfo,
-                             struct LinuxCodecInfo *codecInfo)
+                             struct LinuxCodecInfo *codecInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
     if ((devInfo == nullptr) || (skuTable == nullptr) || (drvInfo == nullptr))
     {
@@ -317,15 +318,17 @@ static bool InitTglMediaSkuExt(struct GfxDeviceInfo *devInfo,
 
      bool compressibleSurfaceEnable = false;
 
-    ReadUserSetting(nullptr,
+    ReadUserSetting(userSettingPtr,
         compressibleSurfaceEnable,
         "Enable Compressible Surface Creation",
         MediaUserSetting::Group::Device);
-
+        
+#ifdef _MMC_SUPPORTED
     if (compressibleSurfaceEnable)
     {
         MEDIA_WR_SKU(skuTable, FtrCompressibleSurfaceDefault, 1);
     }
+#endif
 
     if (drvInfo->devId == 0xFF20)
     {
@@ -341,7 +344,7 @@ static bool InitTglMediaSkuExt(struct GfxDeviceInfo *devInfo,
     //Tile64
     if (GFX_IS_DG2_G11_CONFIG(drvInfo->devId))
     {
-        MEDIA_WR_SKU(skuTable, FtrMediaTile64, 1);
+        MEDIA_WR_SKU(skuTable, FtrTileY, 0);
     }
 
     return true;
@@ -443,9 +446,10 @@ static bool InitTglMediaWaExt(struct GfxDeviceInfo *devInfo,
 
 static bool InitXehpSDVMediaSku(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
-                             struct LinuxDriverInfo *drvInfo)
+                             struct LinuxDriverInfo *drvInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
-    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &XehpSdvCodecInfo))
+    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &XehpSdvCodecInfo, userSettingPtr))
     {
         return false;
     }
@@ -567,9 +571,10 @@ static bool InitXehpSDVMediaWa(struct GfxDeviceInfo *devInfo,
 
 static bool InitPvcMediaSku(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
-                             struct LinuxDriverInfo *drvInfo)
+                             struct LinuxDriverInfo *drvInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
-    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &PvcCodecInfo))
+    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &PvcCodecInfo, userSettingPtr))
     {
         return false;
     }
@@ -739,9 +744,10 @@ static bool InitPvcMediaWa(struct GfxDeviceInfo *devInfo,
 
 static bool InitDg2MediaSku(struct GfxDeviceInfo *devInfo,
                              MediaFeatureTable *skuTable,
-                             struct LinuxDriverInfo *drvInfo)
+                             struct LinuxDriverInfo *drvInfo,
+                             MediaUserSettingSharedPtr userSettingPtr)
 {
-    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &Dg2CodecInfo))
+    if (!InitTglMediaSkuExt(devInfo, skuTable, drvInfo, &Dg2CodecInfo, userSettingPtr))
     {
         return false;
     }
@@ -786,7 +792,7 @@ static bool InitDg2MediaSku(struct GfxDeviceInfo *devInfo,
     MEDIA_WR_SKU(skuTable, FtrDecodeHEVC422VTScalaDisable, 1);
 
     // Tile64
-    if (drvInfo->devRev >=0x04 &&
+    if (drvInfo->devRev >= 0x04 &&
         (drvInfo->devId == 0x4F80 ||
          drvInfo->devId == 0x4F81 ||
          drvInfo->devId == 0x4F82 ||
@@ -804,6 +810,8 @@ static bool InitDg2MediaSku(struct GfxDeviceInfo *devInfo,
          drvInfo->devId == 0x56A0 ||
          drvInfo->devId == 0x56A1 ||
          drvInfo->devId == 0x56A2 ||
+         drvInfo->devId == 0x56A3 ||
+         drvInfo->devId == 0x56A4 ||
          drvInfo->devId == 0x56A5 ||
          drvInfo->devId == 0x56A6 ||
          drvInfo->devId == 0x56A7 ||
@@ -811,11 +819,24 @@ static bool InitDg2MediaSku(struct GfxDeviceInfo *devInfo,
          drvInfo->devId == 0x56A9 ||
          drvInfo->devId == 0x56C0))
     {
-        MEDIA_WR_SKU(skuTable, FtrMediaTile64, 1);
+        MEDIA_WR_SKU(skuTable, FtrTileY, 0);
     }
 
     MEDIA_WR_SKU(skuTable, FtrHDR, 1);
 
+    // Enable HVS Denoise
+    MEDIA_WR_SKU(skuTable, FtrHVSDenoise, 1);
+
+    #define IS_SERVER_SKU(d) (((d) >= 0x56C0) && ((d) <= 0x56C1))
+    if (IS_SERVER_SKU(drvInfo->devId))
+    {
+        drvInfo->isServer = 1;
+    }
+    else
+    {
+        drvInfo->isServer = 0;
+    }
+
     return true;
 }
 
@@ -850,30 +871,10 @@ static bool InitDg2MediaWa(struct GfxDeviceInfo *devInfo,
         MEDIA_WR_WA(waTable, Wa_22011700429, 1);
     }
 
+    // For DG2 128/512 need to distinguish A stepping and B stepping
     // Disable B+ stepping features on DG2A
-    if (drvInfo->devId == 0x4F80 ||
-        drvInfo->devId == 0x4F81 ||
-        drvInfo->devId == 0x4F82 ||
-        drvInfo->devId == 0x4F83 ||
-        drvInfo->devId == 0x4F84 ||
-        drvInfo->devId == 0x4F85 ||
-        drvInfo->devId == 0x4F86 ||
-        drvInfo->devId == 0x5690 ||
-        drvInfo->devId == 0x5691 ||
-        drvInfo->devId == 0x5692 ||
-        drvInfo->devId == 0x5695 ||
-        drvInfo->devId == 0x5696 ||
-        drvInfo->devId == 0x5697 ||
-        drvInfo->devId == 0x5698 ||
-        drvInfo->devId == 0x56A0 ||
-        drvInfo->devId == 0x56A1 ||
-        drvInfo->devId == 0x56A2 ||
-        drvInfo->devId == 0x56A5 ||
-        drvInfo->devId == 0x56A6 ||
-        drvInfo->devId == 0x56A7 ||
-        drvInfo->devId == 0x56A8 ||
-        drvInfo->devId == 0x56A9 ||
-        drvInfo->devId == 0x56C0)
+    if (GFX_IS_DG2_G10_CONFIG(drvInfo->devId) ||
+        GFX_IS_DG2_G11_CONFIG(drvInfo->devId))
     {
         if (drvInfo->devRev < 4)
         {
@@ -886,6 +887,12 @@ static bool InitDg2MediaWa(struct GfxDeviceInfo *devInfo,
         MEDIA_WR_WA(waTable, Wa_22011549751, 1);
     }
 
+    // for DG2 256EU, support B+ features, don't need wa for A stepping
+    if (GFX_IS_DG2_G12_CONFIG(drvInfo->devId))
+    {
+        MEDIA_WR_WA(waTable, Wa_22011549751, 1);
+    }
+
     // CCS Chromakey
     if (GFX_IS_DG2_G11_CONFIG(drvInfo->devId))
     {
@@ -904,6 +911,8 @@ static bool InitDg2MediaWa(struct GfxDeviceInfo *devInfo,
     // Remove the WA of DummyReference
     MEDIA_WR_WA(waTable, WaDummyReference, 0);
 
+    MEDIA_WR_WA(waTable, WaDisableSetObjectCapture, 1);
+
     return true;
 }
 
