@@ -25,7 +25,6 @@
 //!
 #include "encode_hevc_vdenc_pipeline_xe_xpm_base.h"
 #include "encode_utils.h"
-#include "media_user_settings_mgr_g12.h"
 #include "encode_status_report_defs.h"
 #include "encode_scalability_defs.h"
 #include "encode_mem_compression_g12.h"
@@ -34,7 +33,7 @@
 namespace encode {
 
 HevcVdencPipelineXe_Xpm_Base::HevcVdencPipelineXe_Xpm_Base(
-    CodechalHwInterface     *hwInterface,
+    CodechalHwInterfaceNext     *hwInterface,
     CodechalDebugInterface  *debugInterface)
     : HevcVdencPipeline(hwInterface, debugInterface)
 {
@@ -193,18 +192,18 @@ MOS_STATUS HevcVdencPipelineXe_Xpm_Base::Initialize(void *settings)
         if (m_debugInterface != nullptr) {
             MOS_Delete(m_debugInterface);
         }
-        m_debugInterface = MOS_New(CodechalDebugInterfaceG12);
+        m_debugInterface = MOS_New(CodechalDebugInterface);
         ENCODE_CHK_NULL_RETURN(m_debugInterface);
         ENCODE_CHK_STATUS_RETURN(
-            m_debugInterface->Initialize(m_hwInterface, m_codecFunction));
+            m_debugInterface->Initialize(m_hwInterface, m_codecFunction, m_mediaCopy));
 
         if (m_statusReportDebugInterface != nullptr) {
             MOS_Delete(m_statusReportDebugInterface);
         }
-        m_statusReportDebugInterface = MOS_New(CodechalDebugInterfaceG12);
+        m_statusReportDebugInterface = MOS_New(CodechalDebugInterface);
         ENCODE_CHK_NULL_RETURN(m_statusReportDebugInterface);
         ENCODE_CHK_STATUS_RETURN(
-            m_statusReportDebugInterface->Initialize(m_hwInterface, m_codecFunction)););
+            m_statusReportDebugInterface->Initialize(m_hwInterface, m_codecFunction, m_mediaCopy)););
     ENCODE_CHK_STATUS_RETURN(GetSystemVdboxNumber());
 
     return MOS_STATUS_SUCCESS;
@@ -276,7 +275,7 @@ MOS_STATUS HevcVdencPipelineXe_Xpm_Base::InitMmcState()
 #ifdef _MMC_SUPPORTED
     ENCODE_CHK_NULL_RETURN(m_hwInterface);
     m_mmcState = MOS_New(EncodeMemCompG12, m_hwInterface);
-    CODECHAL_ENCODE_CHK_NULL_RETURN(m_mmcState);
+    ENCODE_CHK_NULL_RETURN(m_mmcState);
 #endif
     return MOS_STATUS_SUCCESS;
 }
