@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2019-2021, Intel Corporation
+* Copyright (c) 2019-2022, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -25,27 +25,76 @@
 //!
 #include "decode_av1_pipeline_g12_base.h"
 #include "decode_utils.h"
-#include "media_user_settings_mgr_g12.h"
 #include "codechal_setting.h"
 #include "decode_av1_feature_manager_g12_base.h"
-#include "decode_huc_packet_creator_base.h"
+#include "decode_huc_packet_creator_g12.h"
+#include "decode_sfc_histogram_postsubpipeline_m12.h"
+#include "decode_input_bitstream_m12.h"
+#include "decode_cp_bitstream_m12.h"
+#include "decode_marker_packet_g12.h"
+#include "decode_predication_packet_g12.h"
 
 namespace decode {
 
 Av1PipelineG12_Base::Av1PipelineG12_Base(
     CodechalHwInterface *   hwInterface,
     CodechalDebugInterface *debugInterface)
-    : DecodePipeline(hwInterface, debugInterface)
+    : DecodePipeline(hwInterface ? hwInterface->m_hwInterfaceNext : nullptr, debugInterface)
 {
-
+    m_hwInterface = hwInterface;
 }
 
 MOS_STATUS Av1PipelineG12_Base::Initialize(void *settings)
 {
     DECODE_FUNC_CALL();
-    DECODE_CHK_STATUS(DecodePipeline::Initialize(settings));
+    DECODE_CHK_STATUS(MediaPipeline::InitPlatform());
+    DECODE_CHK_STATUS(MediaPipeline::CreateMediaCopy());
 
-    HucPacketCreatorBase *hucPktCreator = dynamic_cast<HucPacketCreatorBase *>(this);
+    DECODE_CHK_NULL(m_waTable);
+
+    auto *codecSettings = (CodechalSetting *)settings;
+    DECODE_CHK_NULL(m_hwInterface);
+    DECODE_CHK_STATUS(m_hwInterface->Initialize(codecSettings));
+
+    if (m_mediaCopy == nullptr)
+    {
+        m_mediaCopy = m_hwInterface->CreateMediaCopy(m_osInterface);
+    }
+
+#if USE_CODECHAL_DEBUG_TOOL
+    DECODE_CHK_NULL(m_debugInterface);
+    DECODE_CHK_STATUS(m_debugInterface->SetFastDumpConfig(m_mediaCopy));
+#endif
+    if (m_hwInterface->m_hwInterfaceNext)
+    {
+        m_hwInterface->m_hwInterfaceNext->legacyHwInterface = m_hwInterface;
+    }
+    m_mediaContext = MOS_New(MediaContext, scalabilityDecoder, m_hwInterface->m_hwInterfaceNext, m_osInterface);
+    DECODE_CHK_NULL(m_mediaContext);
+
+    m_task = CreateTask(MediaTask::TaskType::cmdTask);
+    DECODE_CHK_NULL(m_task);
+
+    m_numVdbox = GetSystemVdboxNumber();
+
+    bool limitedLMemBar = MEDIA_IS_SKU(m_skuTable, FtrLimitedLMemBar) ? true : false;
+    m_allocator         = MOS_New(DecodeAllocator, m_osInterface, limitedLMemBar);
+    DECODE_CHK_NULL(m_allocator);
+
+    DECODE_CHK_STATUS(CreateStatusReport());
+
+    m_decodecp = Create_DecodeCpInterface(codecSettings, m_hwInterface->GetCpInterface(), m_hwInterface->GetOsInterface());
+    if (m_decodecp)
+    {
+        m_decodecp->RegisterParams(codecSettings);
+    }
+    DECODE_CHK_STATUS(CreateFeatureManager());
+    DECODE_CHK_STATUS(m_featureManager->Init(codecSettings));
+
+    DECODE_CHK_STATUS(CreateSubPipeLineManager(codecSettings));
+    DECODE_CHK_STATUS(CreateSubPacketManager(codecSettings));
+
+    HucPacketCreatorG12 *hucPktCreator = dynamic_cast<HucPacketCreatorG12 *>(this);
     DECODE_CHK_NULL(hucPktCreator);
     m_cdfCopyPkt = hucPktCreator->CreateHucCopyPkt(this, m_task, m_hwInterface); 
     DECODE_CHK_NULL(m_cdfCopyPkt);
@@ -54,21 +103,11 @@ MOS_STATUS Av1PipelineG12_Base::Initialize(void *settings)
     DECODE_CHK_STATUS(RegisterPacket(DecodePacketId(this, defaultCdfBufCopyPacketId), packet));
     DECODE_CHK_STATUS(packet->Init());
 
-    auto *codecSettings = (CodechalSetting*)settings;
-    DECODE_CHK_NULL(codecSettings);
-
-    MOS_USER_FEATURE_VALUE_DATA userFeatureData;
-    MOS_ZeroMemory(&userFeatureData, sizeof(userFeatureData));
-
+  bool forceTileBasedDecodingRead = 0;
 #if (_DEBUG || _RELEASE_INTERNAL)
-    MOS_UserFeature_ReadValue_ID(
-        nullptr,
-        __MEDIA_USER_FEATURE_VALUE_FORCE_AV1_TILE_BASED_DECODE_ID,
-        &userFeatureData,
-        m_osInterface ? m_osInterface->pOsContext : nullptr);
+    forceTileBasedDecodingRead = ReadUserFeature(m_userSettingPtr, "Force Av1 Tile Based Decode", MediaUserSetting::Group::Sequence).Get<bool>();
 #endif
-
-    m_forceTileBasedDecoding = userFeatureData.i32Data;
+    m_forceTileBasedDecoding = forceTileBasedDecodingRead;
 
     return MOS_STATUS_SUCCESS;
 }
@@ -118,13 +157,15 @@ MOS_STATUS Av1PipelineG12_Base::UserFeatureReport()
 MOS_STATUS Av1PipelineG12_Base::ActivateDecodePackets()
 {
     DECODE_FUNC_CALL();
-
+    auto basicFeature    = dynamic_cast<Av1BasicFeatureG12 *>(m_featureManager->GetFeature(FeatureIDs::basicFeature));
+    DECODE_CHK_NULL(basicFeature);
     bool immediateSubmit = true;
 
     if (m_isFirstTileInFrm)
     {
         DECODE_CHK_STATUS(ActivatePacket(DecodePacketId(this, defaultCdfBufCopyPacketId), immediateSubmit, 0, 0));
         m_isFirstTileInFrm = false;
+        m_activePacketList.back().frameTrackingRequested = false;
     }
 
     if (!m_forceTileBasedDecoding)
@@ -135,6 +176,10 @@ MOS_STATUS Av1PipelineG12_Base::ActivateDecodePackets()
     for (uint16_t curPass = 0; curPass < GetPassNum(); curPass++)
     {
         DECODE_CHK_STATUS(ActivatePacket(DecodePacketId(this, av1DecodePacketId), immediateSubmit, curPass, 0));
+        if (basicFeature->m_filmGrainEnabled)
+        {
+            m_activePacketList.back().frameTrackingRequested = false;
+        }
     }
 
     return MOS_STATUS_SUCCESS;
@@ -160,7 +205,7 @@ bool Av1PipelineG12_Base::FrameBasedDecodingInUse()
 MOS_STATUS Av1PipelineG12_Base::CreateFeatureManager()
 {
     DECODE_FUNC_CALL();
-    m_featureManager = MOS_New(DecodeAv1FeatureManagerG12_Base, m_allocator, m_hwInterface);
+    m_featureManager = MOS_New(DecodeAv1FeatureManagerG12_Base, m_allocator, m_hwInterface, m_osInterface);
     DECODE_CHK_NULL(m_featureManager);
     return MOS_STATUS_SUCCESS;
 }
@@ -169,7 +214,15 @@ MOS_STATUS Av1PipelineG12_Base::CreateSubPackets(DecodeSubPacketManager &subPack
 {
     DECODE_FUNC_CALL();
 
-    DECODE_CHK_STATUS(DecodePipeline::CreateSubPackets(subPacketManager, codecSettings));
+    DecodePredicationPktG12 *predicationPkt = MOS_New(DecodePredicationPktG12, this, m_hwInterface);
+    DECODE_CHK_NULL(predicationPkt);
+    DECODE_CHK_STATUS(subPacketManager.Register(
+        DecodePacketId(this, predicationSubPacketId), *predicationPkt));
+
+    DecodeMarkerPktG12 *markerPkt = MOS_New(DecodeMarkerPktG12, this, m_hwInterface);
+    DECODE_CHK_NULL(markerPkt);
+    DECODE_CHK_STATUS(subPacketManager.Register(
+        DecodePacketId(this, markerSubPacketId), *markerPkt));
 
     return MOS_STATUS_SUCCESS;
 }
@@ -179,6 +232,31 @@ Av1PipelineG12_Base::Av1DecodeMode Av1PipelineG12_Base::GetDecodeMode()
     return m_decodeMode;
 }
 
+MOS_STATUS Av1PipelineG12_Base::CreatePostSubPipeLines(DecodeSubPipelineManager &subPipelineManager)
+{
+    DECODE_FUNC_CALL();
+
+#ifdef _DECODE_PROCESSING_SUPPORTED
+    auto sfcHistogramPostSubPipeline = MOS_New(DecodeSfcHistogramSubPipelineM12, this, m_task, m_numVdbox, m_hwInterface);
+    DECODE_CHK_NULL(sfcHistogramPostSubPipeline);
+    DECODE_CHK_STATUS(m_postSubPipeline->Register(*sfcHistogramPostSubPipeline));
+#endif
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS Av1PipelineG12_Base::CreatePreSubPipeLines(DecodeSubPipelineManager &subPipelineManager)
+{
+    m_bitstream = MOS_New(DecodeInputBitstreamM12, this, m_task, m_numVdbox, m_hwInterface);
+    DECODE_CHK_NULL(m_bitstream);
+    DECODE_CHK_STATUS(subPipelineManager.Register(*m_bitstream));
+
+    m_streamout = MOS_New(DecodeStreamOutM12, this, m_task, m_numVdbox, m_hwInterface);
+    DECODE_CHK_NULL(m_streamout);
+    DECODE_CHK_STATUS(subPipelineManager.Register(*m_streamout));
+    return MOS_STATUS_SUCCESS;
+}
+
 #if USE_CODECHAL_DEBUG_TOOL
 MOS_STATUS Av1PipelineG12_Base::DumpParams(Av1BasicFeatureG12 &basicFeature)
 {
@@ -189,12 +267,12 @@ MOS_STATUS Av1PipelineG12_Base::DumpParams(Av1BasicFeatureG12 &basicFeature)
         basicFeature.m_av1PicParams));
 
     DECODE_CHK_STATUS(DumpBitstreamControlParams(
-        basicFeature.m_av1TileParams));
+        basicFeature.m_av1TileParams, basicFeature.m_tileCoding.m_numTiles));
 
     return MOS_STATUS_SUCCESS;
 }
 
-MOS_STATUS Av1PipelineG12_Base::DumpBitstreamControlParams(CodecAv1TileParams *tileParams)
+MOS_STATUS Av1PipelineG12_Base::DumpBitstreamControlParams(CodecAv1TileParams *tileParams, uint32_t tileNum)
 {
     CODECHAL_DEBUG_FUNCTION_ENTER;
 
@@ -208,16 +286,19 @@ MOS_STATUS Av1PipelineG12_Base::DumpBitstreamControlParams(CodecAv1TileParams *t
     std::ostringstream oss;
     oss.setf(std::ios::showbase | std::ios::uppercase);
 
-    oss << "BSTileDataLocation: " << +tileParams->m_bsTileDataLocation << std::endl;
-    oss << "BSTileBytesInBuffer: " << +tileParams->m_bsTileBytesInBuffer << std::endl;
-    oss << "wBadBSBufferChopping: " << +tileParams->m_badBSBufferChopping << std::endl;
-    oss << "tile_row: " << +tileParams->m_tileRow << std::endl;
-    oss << "tile_column: " << +tileParams->m_tileColumn << std::endl;
-    oss << "tile_index: " << +tileParams->m_tileIndex << std::endl;
-    oss << "StartTileIdx: " << +tileParams->m_startTileIdx << std::endl;
-    oss << "EndTileIdx: " << +tileParams->m_endTileIdx << std::endl;
-    oss << "anchor_frame_idx: " << +tileParams->m_anchorFrameIdx.FrameIdx << std::endl;
-    oss << "BSTilePayloadSizeInBytes: " << +tileParams->m_bsTilePayloadSizeInBytes << std::endl;
+    for(uint32_t i = 0; i <  tileNum; i++)
+    {
+        oss << "BSTileDataLocation: " << +(tileParams+i)->m_bsTileDataLocation << std::endl;
+        oss << "BSTileBytesInBuffer: " << +(tileParams+i)->m_bsTileBytesInBuffer << std::endl;
+        oss << "wBadBSBufferChopping: " << +(tileParams+i)->m_badBSBufferChopping << std::endl;
+        oss << "tile_row: " << +(tileParams+i)->m_tileRow << std::endl;
+        oss << "tile_column: " << +(tileParams+i)->m_tileColumn << std::endl;
+        oss << "tile_index: " << +(tileParams+i)->m_tileIndex << std::endl;
+        oss << "StartTileIdx: " << +(tileParams+i)->m_startTileIdx << std::endl;
+        oss << "EndTileIdx: " << +(tileParams+i)->m_endTileIdx << std::endl;
+        oss << "anchor_frame_idx: " << +(tileParams+i)->m_anchorFrameIdx.FrameIdx << std::endl;
+        oss << "BSTilePayloadSizeInBytes: " << +(tileParams+i)->m_bsTilePayloadSizeInBytes << std::endl;
+    }
 
     const char *fileName = m_debugInterface->CreateFileName(
         "DEC",
