@@ -30,7 +30,8 @@
 #include "media_pipeline.h"
 #include "media_cmd_task.h"
 #include "media_packet.h"
-#include "media_interfaces_mcpy.h"
+#include "media_interfaces_mcpy_next.h"
+#include "media_debug_interface.h"
 
 MediaPipeline::MediaPipeline(PMOS_INTERFACE osInterface) : m_osInterface(osInterface)
 {
@@ -51,17 +52,6 @@ MediaPipeline::MediaPipeline(PMOS_INTERFACE osInterface) : m_osInterface(osInter
     {
         perfProfiler->Initialize((void *)this, m_osInterface);
     }
-    //Create both Legacy/APO perf profiler to keep compatability of components/codecs not switching to APO path
-    MediaPerfProfilerNext *perfProfilerNext = MediaPerfProfilerNext::Instance();
-    if (!perfProfilerNext)
-    {
-        MOS_OS_ASSERTMESSAGE("Initialize MediaPerfProfilerNext failed!");
-    }
-    else
-    {
-        perfProfilerNext->Initialize((void *)this, m_osInterface);
-    }
-
 }
 
 MediaPipeline::~MediaPipeline()
@@ -71,7 +61,7 @@ MediaPipeline::~MediaPipeline()
 
     MOS_Delete(m_mediaCopy);
 
-    CODECHAL_DEBUG_TOOL(MOS_Delete(m_debugInterface));
+    MEDIA_DEBUG_TOOL(MOS_Delete(m_debugInterface));
 
     MediaPerfProfiler *perfProfiler = MediaPerfProfiler::Instance();
 
@@ -81,9 +71,12 @@ MediaPipeline::~MediaPipeline()
     }
     else
     {
-        //Destruction of APO perfProfile will be done inside legacy one.
         MediaPerfProfiler::Destroy(perfProfiler, (void *)this, m_osInterface);
     }
+
+#if MHW_HWCMDPARSER_ENABLED
+    mhw::HwcmdParser::DestroyInstance();
+#endif
 }
 
 MOS_STATUS MediaPipeline::DeletePackets()
@@ -121,7 +114,10 @@ MOS_STATUS MediaPipeline::InitPlatform()
 MOS_STATUS MediaPipeline::UserFeatureReport()
 {
 #if (_DEBUG || _RELEASE_INTERNAL)
-    WriteUserFeature(__MEDIA_USER_FEATURE_VALUE_APOGEIOS_ENABLE_ID, 1, m_osInterface->pOsContext);
+    ReportUserSetting(m_userSettingPtr,
+                      __MEDIA_USER_FEATURE_VALUE_APOGEIOS_ENABLE,
+                      uint32_t(1),
+                      MediaUserSetting::Group::Device);
 #endif
     return MOS_STATUS_SUCCESS;
 }
@@ -244,8 +240,6 @@ MediaTask *MediaPipeline::CreateTask(MediaTask::TaskType type)
     case MediaTask::TaskType::cmdTask:
         task = MOS_New(CmdTask, m_osInterface);
         break;
-    case MediaTask::TaskType::mdfTask:
-        break;
     default:
         break;
     }
@@ -290,7 +284,7 @@ MOS_STATUS MediaPipeline::CreateMediaCopy()
     {
         m_osInterface->pfnGetMosContext(m_osInterface, &mos_context);
     }
-    m_mediaCopy = static_cast<MediaCopyBaseState*>(McpyDevice::CreateFactory(mos_context));
+    m_mediaCopy = static_cast<MediaCopyBaseState *>(McpyDeviceNext::CreateFactory((MOS_CONTEXT_HANDLE)mos_context));
     return MOS_STATUS_SUCCESS;
 }
 
