@@ -55,6 +55,24 @@ class SwFilterSubPipe;
 #define FEATURE_TYPE_MASK                   0xffffff00
 #define FEATURE_TYPE_ENGINE_ASSIGNED(feature) (((feature)&FEATURE_TYPE_MASK) != (feature))
 
+#define SURFACETYPE_SIZE32                  31
+#define SURFACETYPE_SIZE16                  15
+#define SURFACETYPE_SIZE5                   4
+#define SURFACETYPE_SIZE10                  9
+
+#define GFSURFACE_COUNT 8
+#define MOTION_CHANNEL 3
+#define MAX_MODELSURFACE_COUNT 85
+
+#define VPHAL_MAX_HDR_INPUT_LAYER           8
+#define VPHAL_MAX_HDR_OUTPUT_LAYER          1
+#define VPHAL_HDR_BTINDEX_LAYER0            16
+#define VPHAL_HDR_BTINDEX_PER_LAYER0        5
+#define VPHAL_HDR_BTINDEX_RENDERTARGET      56
+#define VPHAL_HDR_BTINDEX_PER_TARGET        3
+#define VPHAL_HDR_SAMPLER8X8_TABLE_NUM      2
+#define ARRAY_SIZE(a)  (sizeof(a) / sizeof(a[0]))
+
 enum FeatureType
 {
     FeatureTypeInvalid          = 0,
@@ -91,12 +109,14 @@ enum FeatureType
     FeatureTypeHdr              = 0xC00,
     FeatureTypeHdrOnVebox       = FeatureTypeHdr | FEATURE_TYPE_ENGINE_BITS_VEBOX,
     FeatureTypeHdr3DLutCalOnRender = FeatureTypeHdr | FEATURE_TYPE_ENGINE_BITS_RENDER | FEATURE_TYPE_ENGINE_BITS_SUB_STEP,
+    FeatureTypeHdrOnRender      = FeatureTypeHdr | FEATURE_TYPE_ENGINE_BITS_RENDER,
     FeatureTypeFD               = 0xD00,
     FeatureTypeFLD              = 0xE00,
     FeatureTypeFB               = 0xF00,
     FeatureTypeSecureCopy       = 0x1000,
     FeatureTypeSecureCopyOnRender = FeatureTypeSecureCopy | FEATURE_TYPE_ENGINE_BITS_RENDER,
     FeatureTypeSR               = 0x1100,
+    FeatureTypeSROnVebox        = FeatureTypeSR | FEATURE_TYPE_ENGINE_BITS_VEBOX,
     FeatureTypeSROnRender       = FeatureTypeSR | FEATURE_TYPE_ENGINE_BITS_RENDER,
     FeatureTypeLace             = 0x1200,
     FeatureTypeLaceOnVebox      = FeatureTypeLace | FEATURE_TYPE_ENGINE_BITS_VEBOX,
@@ -123,6 +143,10 @@ enum FeatureType
     FeatureTypeFDFB             = 0x2100,
     FeatureTypeFDFBOnVebox      = FeatureTypeFDFB | FEATURE_TYPE_ENGINE_BITS_VEBOX,
     FeatureTypeFDFBOnRender     = FeatureTypeFDFB | FEATURE_TYPE_ENGINE_BITS_RENDER,
+    FeatureTypeSegmentation     = 0x2200,
+    FeatureTypeSegmentationOnVebox  = FeatureTypeSegmentation | FEATURE_TYPE_ENGINE_BITS_VEBOX,
+    FeatureTypeSegmentationOnRender = FeatureTypeSegmentation | FEATURE_TYPE_ENGINE_BITS_RENDER,
+    FeatureTypeS3D                  = 0x2300,
     // ...
     NumOfFeatureType
 };
@@ -193,6 +217,7 @@ enum SurfaceType
     SurfaceTypeFcTarget0,
     SurfaceTypeFcTarget1,
     SurfaceTypeFcCscCoeff,
+    SurfaceTypeDecompressionSync,
     // LGCA related Surfaces
     SurfaceTypeSamplerSurfaceR,
     SurfaceTypeSamplerSurfaceG,
@@ -624,11 +649,164 @@ enum SurfaceType
     SurfaceTypeFBInputSampler,
     // HVS Kernel
     SurfaceTypeHVSTable,
+    //Segmentation
+    SurfaceTypeSegRenderPreviousInput,
+    SurfaceTypeSegRenderTempOutput, //Used for seg out only
+    SurfaceTypeSegColorBalanceInput,
+    SurfaceTypeSegBackground,
+    SurfaceTypeSegGaussianCoeffBuffer,
+    SurfaceTypeSegTFMask,
+    SurfaceTypeSegTFOutMask,
+    SurfaceTypeSegWindowSize,
+    SurfaceTypeSegMaskThreshold,
+    SurfaceTypeSegMaskScaling,
+    SurfaceTypeSegMaskBlur,
+    SurfaceTypeSegBlurScaling,
+    SurfaceTypeSegGaussianBlur,
+    SurfaceTypeSegBlur,
+    SurfaceTypeSegBlur2,
+    SurfaceTypeSegBlur3,
+    SurfaceTypeSegGFLayer,
+    SurfaceTypeSegGFLayerEnd = SurfaceTypeSegGFLayer + GFSURFACE_COUNT,
+    SurfaceTypeSegGFOut,
+    SurfaceTypeSegGFOut2,
+    SurfaceTypeSegGFOut3,
+    SurfaceTypeSegGFOut4,
+    SurfaceTypeSegInputMotion,
+    SurfaceTypeSegInputMotionEnd = SurfaceTypeSegInputMotion + MOTION_CHANNEL - 1,
+    SurfaceTypeSegPreInputMotion,
+    SurfaceTypeSegPreInputMotionEnd = SurfaceTypeSegPreInputMotion + MOTION_CHANNEL - 1,
+    SurfaceTypeSegDiffMotion,
+    SurfaceTypeSegDiffMotionEnd = SurfaceTypeSegDiffMotion + MOTION_CHANNEL - 1,
+    SurfaceTypeSegErode1x2Motion,
+    SurfaceTypeSegErode1x2MotionEnd = SurfaceTypeSegErode1x2Motion + MOTION_CHANNEL - 1,
+    SurfaceTypeSegErode2x1Motion,
+    SurfaceTypeSegErode2x1Motion2,
+    SurfaceTypeSegErode2x1Motion3,
+    SurfaceTypeSegSumMotion,
+    SurfaceTypeSegRemoveBlob,
+    SurfaceTypeSegColorBalance,
+    // Segmentation layers
+    SurfaceTypeSegModelLayer,
+    SurfaceTypeSegModelLayerEnd = SurfaceTypeSegModelLayer + MAX_MODELSURFACE_COUNT,
+    SurfaceTypeSegWeights,
+    SurfaceTypeSegWeightsEnd = SurfaceTypeSegWeights + MAX_MODELSURFACE_COUNT,
+    SurfaceTypeSegBias,
+    SurfaceTypeSegBiasEnd = SurfaceTypeSegBias + MAX_MODELSURFACE_COUNT,
+
+    // SR
+    SurfaceTypeSRLumaInputSurf,
+    SurfaceTypeSRTempOutputSurf,
+    SurfaceTypeSRTempLumaOutput,
+    SurfaceTypeSRTempLumaOutputEnd = SurfaceTypeSRTempLumaOutput + SURFACETYPE_SIZE32,
+    SurfaceTypeSRTempLuma2Output,
+    SurfaceTypeSRTempLuma2OutputEnd = SurfaceTypeSRTempLuma2Output + SURFACETYPE_SIZE16,
+    SurfaceTypeSRTempLuma3Output,
+    SurfaceTypeSRTempLuma3OutputEnd = SurfaceTypeSRTempLuma3Output + SURFACETYPE_SIZE5,
+    SurfaceTypeSRTempLuma4Output,
+    SurfaceTypeSRTempLuma4OutputEnd = SurfaceTypeSRTempLuma4Output + SURFACETYPE_SIZE32,
+    SurfaceTypeSRTempLuma5Output,
+    SurfaceTypeSRWeightBuffer,
+    SurfaceTypeSRWeightBufferEnd = SurfaceTypeSRWeightBuffer + SURFACETYPE_SIZE10,
+    SurfaceTypeSRBiasBuffer,
+    SurfaceTypeSRBiasBufferEnd = SurfaceTypeSRBiasBuffer + SURFACETYPE_SIZE10,
+    SurfaceTypeSRPreluBuffer,
+    SurfaceTypeSRPreluBufferEnd = SurfaceTypeSRPreluBuffer + SURFACETYPE_SIZE10,
+    SurfaceTypeSRSlopeBuffer,
+    SurfaceTypeSRSlopeBufferEnd = SurfaceTypeSRSlopeBuffer + SURFACETYPE_SIZE5,
+    SurfaceTypeSROutput,
+    SurfaceTypeSRChromaInput,
+
+    // HDR Kernel
+    SurfaceTypeHdrInputLayer0,
+    SurfaceTypeHdrInputLayer1,
+    SurfaceTypeHdrInputLayer2,
+    SurfaceTypeHdrInputLayer3,
+    SurfaceTypeHdrInputLayer4,
+    SurfaceTypeHdrInputLayer5,
+    SurfaceTypeHdrInputLayer6,
+    SurfaceTypeHdrInputLayer7,
+    SurfaceTypeHdrInputLayerMax = SurfaceTypeHdrInputLayer7,
+    SurfaceTypeHdrOETF1DLUTSurface0,
+    SurfaceTypeHdrOETF1DLUTSurface1,
+    SurfaceTypeHdrOETF1DLUTSurface2,
+    SurfaceTypeHdrOETF1DLUTSurface3,
+    SurfaceTypeHdrOETF1DLUTSurface4,
+    SurfaceTypeHdrOETF1DLUTSurface5,
+    SurfaceTypeHdrOETF1DLUTSurface6,
+    SurfaceTypeHdrOETF1DLUTSurface7,
+    SurfaceTypeHdrCRI3DLUTSurface0,
+    SurfaceTypeHdrCRI3DLUTSurface1,
+    SurfaceTypeHdrCRI3DLUTSurface2,
+    SurfaceTypeHdrCRI3DLUTSurface3,
+    SurfaceTypeHdrCRI3DLUTSurface4,
+    SurfaceTypeHdrCRI3DLUTSurface5,
+    SurfaceTypeHdrCRI3DLUTSurface6,
+    SurfaceTypeHdrCRI3DLUTSurface7,
+    SurfaceTypeHdrTarget0,
+    SurfaceTypeHdrCoeff,
+    SurfaceTypeHdrAutoModeCoeff,
+    SurfaceTypeHdrAutoModeIirTempSurface,
     NumberOfSurfaceType
 };
 
 using  VP_SURFACE_GROUP = std::map<SurfaceType, VP_SURFACE*>;
 
+enum OutMode
+{
+    OUT_DISABLED = 0,
+    OUT_TO_TARGET,
+    OUT_TO_NEXTPASS
+};
+
+struct REMOVE_BB_SETTING
+{
+    bool     isRemoveBB    = false;
+    bool     isKeepMaxBlob = false;
+    uint32_t index                          = 0;
+    uint32_t height                         = 0;
+    uint32_t width                          = 0;
+    uint32_t size                           = 0;
+    uint16_t inputActiveRegionWidth         = 0;
+    uint16_t inputActiveRegionHeight        = 0;
+    uint8_t *removeBlobLinearAddressAligned = 0;
+};
+
+struct MOTIONLESS_SETTING
+{
+    bool     isEnable        = false;
+    bool     isSkipDetection = false;
+    bool     isInfer         = false;
+    bool     isFirstConv     = false;
+    bool     isMotion        = false;
+    bool     isResUpdate     = false;
+    bool     forceDisable    = false;
+    uint32_t width           = 0;
+    uint32_t height          = 0;
+};
+
+struct COLOR_BALANCE_SETTING
+{
+    bool     isEnable                 = false;
+    uint32_t index                    = 0;
+    uint32_t skipThreshold            = 0;
+    uint16_t inputActiveRegionWidth   = 0;
+    uint16_t inputActiveRegionHeight  = 0;
+    uint32_t pitch                    = 0;
+    uint32_t mPitch                   = 0;
+    uint8_t  downScaleFactor          = 1;
+    uint8_t  colorTemperature         = 0;
+    uint8_t *cbLinearAddressAligned   = 0;
+    double   backgroundWhiteMatrix[3] = {1, 1, 1};
+};
+
+struct VP_POSTPROCESS_SURFACE
+{
+    REMOVE_BB_SETTING  removeBBSetting;
+    MOTIONLESS_SETTING motionlessSetting;
+    COLOR_BALANCE_SETTING colorBalanceSetting;
+};
+
 struct VP_SURFACE_SETTING
 {
     VP_SURFACE_GROUP    surfGroup;
@@ -644,6 +822,10 @@ struct VP_SURFACE_SETTING
     bool                dumpLaceSurface                        = false;
     bool                dumpPreSurface                         = false;
     bool                dumpPostSurface                        = false;
+    VP_POSTPROCESS_SURFACE postProcessSurface                  = {};
+    const uint16_t     *pHDRStageConfigTable                   = nullptr;
+    bool                coeffAllocated                         = false;
+    bool                OETF1DLUTAllocated                     = false;
 
     void Clean()
     {
@@ -660,6 +842,12 @@ struct VP_SURFACE_SETTING
         dumpLaceSurface                        = false;
         dumpPreSurface                         = false;
         dumpPostSurface                        = false;
+        postProcessSurface.removeBBSetting     = {};
+        postProcessSurface.motionlessSetting   = {};
+        postProcessSurface.colorBalanceSetting = {};
+        pHDRStageConfigTable                   = nullptr;
+        coeffAllocated                         = false;
+        OETF1DLUTAllocated                     = false;
     }
 };
 
@@ -808,7 +996,7 @@ protected:
     RenderTargetType m_renderTargetType = RenderTargetTypeSurface;
     bool m_isInExePipe = false;
 
-MEDIA_CLASS_DEFINE_END(SwFilter)
+MEDIA_CLASS_DEFINE_END(vp__SwFilter)
 };
 
 struct FeatureParamCsc : public FeatureParam
@@ -817,11 +1005,16 @@ struct FeatureParamCsc : public FeatureParam
     {
         VPHAL_CSPACE    colorSpace      = CSpace_None;
         uint32_t        chromaSiting    = 0;
+        bool operator == (const struct CSC_PARAMS &b)
+        {
+            return colorSpace == b.colorSpace && chromaSiting == b.chromaSiting;
+        }
     };
     CSC_PARAMS          input           = {};
     CSC_PARAMS          output          = {};
     PVPHAL_IEF_PARAMS   pIEFParams      = nullptr;
     PVPHAL_ALPHA_PARAMS pAlphaParams    = nullptr;
+    MOS_FORMAT          formatforCUS    = Format_None;            //!< Only valid when formatforCUS != Format_None for Chromaupsampling. To be cleared in SwFilterCsc::Configure
     FeatureParamCsc     *next           = nullptr;                //!< pointe to new/next generated CSC params
 };
 
@@ -844,7 +1037,7 @@ public:
 private:
     FeatureParamCsc m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterCsc)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterCsc)
 };
 
 struct FeatureParamScaling : public FeatureParam
@@ -857,6 +1050,12 @@ struct FeatureParamScaling : public FeatureParam
         RECT                    rcDst    = {0, 0, 0, 0};  //!< Input dst rect without rotate being applied.
         RECT                    rcMaxSrc = {0, 0, 0, 0};
         VPHAL_SAMPLE_TYPE       sampleType = SAMPLE_PROGRESSIVE;
+        MOS_TILE_MODE_GMM       tileMode   = MOS_TILE_4_GMM;
+        bool operator == (struct SCALING_PARAMS &b)
+        {
+            // no use sizeof(SCALING_PARAMS) to avoid undefined padding data being used.
+            return 0 == memcmp(this, &b, (uint64_t)(&tileMode) - (uint64_t)(this) + sizeof(tileMode));
+        }
     };
 
     // Parameters maintained by scaling feature parameters
@@ -880,6 +1079,28 @@ struct FeatureParamScaling : public FeatureParam
         bool                    rotationNeeded = false;                 //!< Whether rotate SwFilter exists on SwFilterPipe.
     } rotation;
 
+    bool operator == (struct FeatureParamScaling &b)
+    {
+        return formatInput          == b.formatInput            &&
+            formatOutput            == b.formatOutput           &&
+            input                   == b.input                  &&
+            output                  == b.output                 &&
+            isPrimary               == b.isPrimary              &&
+            scalingMode             == b.scalingMode            &&
+            scalingPreference       == b.scalingPreference      &&
+            bDirectionalScalar      == b.bDirectionalScalar     &&
+            bTargetRectangle        == b.bTargetRectangle       &&
+            interlacedScalingType   == b.interlacedScalingType  &&
+            csc.colorSpaceOutput    == b.csc.colorSpaceOutput   &&
+            rotation.rotationNeeded == b.rotation.rotationNeeded &&
+            (nullptr == pColorFillParams    && nullptr == b.pColorFillParams ||
+            nullptr != pColorFillParams     && nullptr != b.pColorFillParams &&
+            0 == memcmp(pColorFillParams, b.pColorFillParams, sizeof(VPHAL_COLORFILL_PARAMS))) &&
+            (nullptr == pCompAlpha          && nullptr == b.pCompAlpha ||
+            nullptr != pCompAlpha           && nullptr != b.pCompAlpha &&
+            0 == memcmp(pCompAlpha, b.pCompAlpha, sizeof(VPHAL_ALPHA_PARAMS)));
+    }
+
     FeatureParamScaling        *next = nullptr;                           //!< pointe to new/next generated scaling params
 };
 
@@ -906,7 +1127,7 @@ public:
 private:
     FeatureParamScaling m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterScaling)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterScaling)
 };
 
 struct FeatureParamRotMir : public FeatureParam
@@ -918,6 +1139,12 @@ struct FeatureParamRotMir : public FeatureParam
     struct {
         MOS_TILE_TYPE tileOutput = MOS_TILE_X;
     } surfInfo;
+
+    bool operator == (const struct FeatureParamRotMir &b)
+    {
+        return rotation == b.rotation &&
+            surfInfo.tileOutput == b.surfInfo.tileOutput;
+    }
 };
 
 class SwFilterRotMir : public SwFilter
@@ -936,7 +1163,7 @@ public:
 private:
     FeatureParamRotMir m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterRotMir)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterRotMir)
 };
 
 enum DN_STAGE
@@ -956,6 +1183,17 @@ struct FeatureParamDenoise : public FeatureParam
     uint32_t             heightInput          = 0;
     bool                 secureDnNeeded       = false;
     DN_STAGE             stage                = DN_STAGE_DEFAULT;
+    uint32_t             srcBottom            = 0;
+    bool                 operator==(const struct FeatureParamDenoise &b)
+    {
+        return sampleTypeInput     == b.sampleTypeInput &&
+               denoiseParams       == b.denoiseParams   &&
+               widthAlignUnitInput == b.widthAlignUnitInput &&
+               heightAlignUnitInput == b.heightAlignUnitInput &&
+               MOS_MIN(heightInput, srcBottom) == MOS_MIN(b.heightInput, srcBottom) &&
+               secureDnNeeded      == b.secureDnNeeded &&
+               stage               == b.stage;
+    }
 };
 
 class SwFilterDenoise : public SwFilter
@@ -965,6 +1203,7 @@ public:
     virtual ~SwFilterDenoise();
     virtual MOS_STATUS Clean();
     virtual MOS_STATUS Configure(VP_PIPELINE_PARAMS& params, bool isInputSurf, int surfIndex);
+    virtual MOS_STATUS Configure(FeatureParamDenoise &params);
     virtual FeatureParamDenoise& GetSwFilterParams();
     virtual SwFilter* Clone();
     virtual bool operator == (SwFilter& swFilter);
@@ -979,7 +1218,7 @@ public:
 private:
     FeatureParamDenoise m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterDenoise)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterDenoise)
 };
 
 struct FeatureParamDeinterlace : public FeatureParam
@@ -1015,13 +1254,17 @@ public:
 private:
     FeatureParamDeinterlace m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterDeinterlace)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterDeinterlace)
 };
 
 struct FeatureParamSte : public FeatureParam
 {
-    bool       bEnableSTE  = false;
-    uint32_t   dwSTEFactor = 0;
+    bool              bEnableSTE  = false;
+    uint32_t          dwSTEFactor = 0;
+    
+    // For STD alone case
+    bool              bEnableSTD  = false;
+    VPHAL_STD_PARAMS  STDParam    = {};
 };
 
 class SwFilterSte : public SwFilter
@@ -1035,11 +1278,16 @@ public:
     virtual SwFilter* Clone();
     virtual bool operator == (SwFilter& swFilter);
     virtual MOS_STATUS Update(VP_SURFACE* inputSurf, VP_SURFACE* outputSurf, SwFilterSubPipe &pipe);
+    virtual MOS_STATUS SetResourceAssignmentHint(RESOURCE_ASSIGNMENT_HINT &hint)
+    {
+        hint.isSkinScoreDumpNeededForSTDonly = m_Params.bEnableSTD;
+        return MOS_STATUS_SUCCESS;
+    }
 
 private:
     FeatureParamSte m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterSte)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterSte)
 };
 
 struct FeatureParamTcc : public FeatureParam
@@ -1068,7 +1316,7 @@ public:
 private:
     FeatureParamTcc m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterTcc)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterTcc)
 };
 
 struct FeatureParamProcamp : public FeatureParam
@@ -1091,7 +1339,7 @@ public:
 private:
     FeatureParamProcamp m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterProcamp)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterProcamp)
 };
 
 enum HDR_STAGE
@@ -1102,14 +1350,73 @@ enum HDR_STAGE
     HDR_STAGE_VEBOX_3DLUT_NO_UPDATE,
 };
 
+//!
+//! \brief Hdr stages enable flag
+//!
+typedef union _HDRStageEnables
+{
+    uint16_t value;
+    struct
+    {
+        uint16_t PriorCSCEnable : 1;
+        uint16_t EOTFEnable : 1;
+        uint16_t CCMEnable : 1;
+        uint16_t PWLFEnable : 1;
+        uint16_t CCMExt1Enable : 1;
+        uint16_t GamutClamp1Enable : 1;
+        uint16_t CCMExt2Enable : 1;
+        uint16_t GamutClamp2Enable : 1;
+        uint16_t OETFEnable : 1;
+        uint16_t PostCSCEnable : 1;
+        uint16_t Reserved : 6;
+    };
+} HDRStageEnables, *PHDRStageEnables;
+
+//!
+//! Structure VPHAL_HDR_PARAMS
+//! \brief High Dynamic Range parameters
+//!
+typedef struct _HDR_PARAMS
+{
+    VPHAL_HDR_EOTF_TYPE EOTF                 = VPHAL_HDR_EOTF_INVALID;    //!< Electronic-Optimal Transfer Function
+    uint16_t display_primaries_x[3]          = {0};                       //!< Display Primaries X chromaticity coordinates
+    uint16_t display_primaries_y[3]          = {0};                       //!< Display Primaries Y chromaticity coordinates
+    uint16_t white_point_x                   = 0;                         //!< X Chromaticity coordinate of White Point
+    uint16_t white_point_y                   = 0;                         //!< Y Chromaticity coordinate of White Point
+    uint16_t max_display_mastering_luminance = 0;                         //!< The nominal maximum display luminance of the mastering display
+    uint16_t min_display_mastering_luminance = 0;                         //!< The nominal minimum display luminance of the mastering display
+    uint16_t MaxCLL                          = 0;                         //!< Max Content Light Level
+    uint16_t MaxFALL                         = 0;                         //!< Max Frame Average Light Level
+    bool     bAutoMode                       = false;                     //!< Hdr auto mode.
+    bool     bPathKernel                     = false;                     //!< Hdr path config to use kernel
+} HDR_PARAMS, *PHDR_PARAMS;
+
 struct FeatureParamHdr : public FeatureParam
 {
-    uint32_t        uiMaxDisplayLum      = 0;  //!< Maximum Display Luminance
-    uint32_t        uiMaxContentLevelLum = 0;  //!< Maximum Content Level Luminance
-    VPHAL_HDR_MODE  hdrMode              = VPHAL_HDR_MODE_NONE;
-    VPHAL_CSPACE    srcColorSpace        = CSpace_None;
-    VPHAL_CSPACE    dstColorSpace        = CSpace_None;
-    HDR_STAGE       stage                = HDR_STAGE_DEFAULT;
+    uint32_t           uiMaxDisplayLum                                      = 0;                   //!< Maximum Display Luminance
+    uint32_t           uiMaxContentLevelLum                                 = 0;                   //!< Maximum Content Level Luminance
+    VPHAL_HDR_MODE     hdrMode                                              = VPHAL_HDR_MODE_NONE;
+    VPHAL_CSPACE       srcColorSpace                                        = CSpace_None;
+    VPHAL_CSPACE       dstColorSpace                                        = CSpace_None;
+    HDR_STAGE          stage                                                = HDR_STAGE_DEFAULT;
+    uint32_t           widthInput                                           = 0;
+    uint32_t           heightInput                                          = 0;
+    VPHAL_HDR_LUT_MODE LUTMode[VPHAL_MAX_HDR_INPUT_LAYER]                   = {};                  //!< LUT Mode
+    VPHAL_HDR_LUT_MODE GlobalLutMode                                        = {};                  //!< Global LUT mode control for debugging purpose
+    bool               bGpuGenerate3DLUT                                    = false;               //!< Flag for per frame GPU generation of 3DLUT
+    uint32_t           uSourceCount                                         = 0;                   //!< Number of sources
+    uint32_t           uTargetCount                                         = 0;                   //!< Number of targets
+
+    PVPHAL_COLORFILL_PARAMS pColorFillParams                     = nullptr;               //!< ColorFill - BG only
+    bool                    bDisableAutoMode                     = false;                 //!< Force to disable Hdr auto mode tone mapping for debugging purpose
+    uint32_t                uiSplitFramePortions                 = 1;                     //!< Split Frame flag
+    bool                    bForceSplitFrame                     = false;
+    bool                    bNeed3DSampler                       = false;                 //!< indicate whether 3D should neede by force considering AVS removal etc.
+
+    uint16_t         InputSrc[VPHAL_MAX_HDR_INPUT_LAYER]                 = {}; // Input Surface
+    uint16_t         Target[VPHAL_MAX_HDR_OUTPUT_LAYER]                  = {}; // Target Surface
+    HDR_PARAMS       srcHDRParams[VPHAL_MAX_HDR_INPUT_LAYER]             = {};
+    HDR_PARAMS       targetHDRParams[VPHAL_MAX_HDR_OUTPUT_LAYER]         = {};
 };
 
 class SwFilterHdr : public SwFilter
@@ -1130,10 +1437,18 @@ public:
         return MOS_STATUS_SUCCESS;
     }
 
+    MOS_STATUS HdrIsInputFormatSupported(
+        PVPHAL_SURFACE pSrcSurface,
+        bool          *pbSupported);
+
+    MOS_STATUS HdrIsOutputFormatSupported(
+            PVPHAL_SURFACE pTargetSurface,
+            bool          *pbSupported);
+
 private:
     FeatureParamHdr m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterHdr)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterHdr)
 };
 
 struct FeatureParamLumakey : public FeatureParam
@@ -1156,7 +1471,7 @@ public:
 private:
     FeatureParamLumakey m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterLumakey)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterLumakey)
 };
 
 struct FeatureParamBlending : public FeatureParam
@@ -1179,12 +1494,18 @@ public:
 private:
     FeatureParamBlending m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterBlending)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterBlending)
 };
 
 struct FeatureParamColorFill : public FeatureParam
 {
     PVPHAL_COLORFILL_PARAMS colorFillParams = nullptr;     //!< ColorFill - BG only
+    bool operator == (const struct FeatureParamColorFill &b)
+    {
+        return (nullptr == colorFillParams && nullptr == b.colorFillParams ||
+            nullptr != colorFillParams && nullptr != b.colorFillParams &&
+            0 == memcmp(colorFillParams, b.colorFillParams, sizeof(*b.colorFillParams)));
+    }
 };
 
 class SwFilterColorFill : public SwFilter
@@ -1203,13 +1524,20 @@ public:
 private:
     FeatureParamColorFill m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterColorFill)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterColorFill)
 };
 
 struct FeatureParamAlpha : public FeatureParam
 {
     PVPHAL_ALPHA_PARAMS     compAlpha         = nullptr;      //!< Alpha for composited surface
     bool                    calculatingAlpha  = false;        //!< Alpha calculation parameters
+    bool operator == (const struct FeatureParamAlpha &b)
+    {
+        return calculatingAlpha == b.calculatingAlpha &&
+            (nullptr == compAlpha   && nullptr == b.compAlpha ||
+            nullptr != compAlpha    && nullptr != b.compAlpha &&
+            0 == memcmp(compAlpha, b.compAlpha, sizeof(*b.compAlpha)));
+    }
 };
 
 class SwFilterAlpha : public SwFilter
@@ -1227,7 +1555,43 @@ public:
 private:
     FeatureParamAlpha m_Params = {};
 
-MEDIA_CLASS_DEFINE_END(SwFilterAlpha)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterAlpha)
+};
+
+struct FeatureParamCgc : public FeatureParam
+{
+    VPHAL_GAMUT_MODE    GCompMode           = GAMUT_MODE_NONE;
+    VPHAL_CSPACE        colorSpace          = CSpace_None;
+    VPHAL_CSPACE        dstColorSpace       = CSpace_None;
+    bool                bBt2020ToRGB        = false;
+    bool                bExtendedSrcGamut   = false;
+    bool                bExtendedDstGamut   = false;
+    uint32_t            dwAttenuation       = 0;                //!< U2.10 [0, 1024] 0 = No down scaling, 1024 = Full down scaling
+    float               displayRGBW_x[4]    = {};
+    float               displayRGBW_y[4]    = {};
+};
+
+class SwFilterCgc : public SwFilter
+{
+public:
+    SwFilterCgc(VpInterface& vpInterface);
+    virtual ~SwFilterCgc();
+    virtual MOS_STATUS Clean();
+    virtual MOS_STATUS Configure(VP_PIPELINE_PARAMS& params, bool isInputSurf, int surfIndex);
+    virtual FeatureParamCgc& GetSwFilterParams();
+    virtual SwFilter* Clone();
+    virtual bool operator == (SwFilter& swFilter);
+    virtual MOS_STATUS Update(VP_SURFACE* inputSurf, VP_SURFACE* outputSurf, SwFilterSubPipe &pipe);
+    virtual bool IsBt2020ToRGB(VP_PIPELINE_PARAMS& params, bool isInputSurf, int surfIndex);
+    virtual bool IsBt2020ToRGBEnabled()
+    {
+        return m_Params.bBt2020ToRGB;
+    }
+
+protected:
+    FeatureParamCgc m_Params = {};
+
+MEDIA_CLASS_DEFINE_END(vp__SwFilterCgc)
 };
 
 class SwFilterSet
@@ -1255,7 +1619,7 @@ private:
     // nullptr if it is unordered filters, otherwise, it's the pointer to m_OrderedFilters it belongs to.
     std::vector<class SwFilterSet *> *m_location = nullptr;
 
-MEDIA_CLASS_DEFINE_END(SwFilterSet)
+MEDIA_CLASS_DEFINE_END(vp__SwFilterSet)
 };
 
 }
