@@ -296,15 +296,15 @@ MOS_STATUS SwFilterScalingHandler::UpdateParamsForProcessing(VP_PIPELINE_PARAMS&
 {
     VP_FUNC_CALL();
 
-    if (index >= GetPipeCountForProcessing(params))
-    {
-        return MOS_STATUS_INVALID_PARAMETER;
-    }
-
     // For second submission of field-to-interleaved mode, we will take second field as input surface,
     // second field is stored in pBwdRef.
     if (params.pSrc[0] && params.pSrc[0]->InterlacedScalingType == ISCALING_FIELD_TO_INTERLEAVED && index == 1)
     {
+        if (index >= GetPipeCountForProcessing(params))
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        }
+
         if (params.pSrc[0] && params.pSrc[0]->pBwdRef)
         {
             params.pSrc[0]->pBwdRef->ScalingMode = params.pSrc[0]->ScalingMode;
@@ -356,6 +356,30 @@ bool SwFilterDnHandler::IsFeatureEnabled(VP_PIPELINE_PARAMS& params, bool isInpu
 {
     VP_FUNC_CALL();
 
+    PVP_MHWINTERFACE hwInterface = m_vpInterface.GetHwInterface();
+    // secure mode
+    if (hwInterface->m_osInterface->osCpInterface &&
+        hwInterface->m_osInterface->osCpInterface->IsHMEnabled())
+    {
+        VP_PUBLIC_NORMALMESSAGE("Dn is disabled in secure mode.");
+        return false;
+    }
+    // clear mode
+    else
+    {
+        auto userFeatureControl = hwInterface->m_userFeatureControl;
+        if (userFeatureControl != nullptr)
+        {
+            bool disableDn = userFeatureControl->IsDisableDn();
+            if (disableDn)
+            {
+                VP_PUBLIC_NORMALMESSAGE("Dn is disabled in clear mode.");
+                return false;
+            }
+        }
+       
+    }
+
     if (!SwFilterFeatureHandler::IsFeatureEnabled(params, isInputSurf, surfIndex, pipeType))
     {
         return false;
@@ -436,7 +460,7 @@ bool SwFilterDiHandler::IsFeatureEnabled(VP_PIPELINE_PARAMS& params, bool isInpu
         vphalSurf->Format != Format_P010 &&
         vphalSurf->Format != Format_P016)
     {
-        VPHAL_PUBLIC_NORMALMESSAGE("Query Variance is enabled, but APG didn't support this feature yet");
+        VP_PUBLIC_NORMALMESSAGE("Query Variance is enabled, but APG didn't support this feature yet");
     }
 
     return false;
@@ -489,7 +513,8 @@ bool SwFilterSteHandler::IsFeatureEnabled(VP_PIPELINE_PARAMS& params, bool isInp
 
     PVPHAL_SURFACE vphalSurf = isInputSurf ? params.pSrc[surfIndex] : params.pTarget[surfIndex];
     if (vphalSurf && vphalSurf->pColorPipeParams &&
-        vphalSurf->pColorPipeParams->bEnableSTE)
+            (vphalSurf->pColorPipeParams->bEnableSTE ||
+        vphalSurf->pColorPipeParams->bEnableSTD))
     {
         return true;
     }
@@ -929,3 +954,81 @@ void SwFilterAlphaHandler::Destory(SwFilter*& swFilter)
     m_swFilterFactory.Destory(filter);
     return;
 }
+
+/****************************************************************************************************/
+/*                                      SwFilterCgcHandler                                          */
+/****************************************************************************************************/
+
+SwFilterCgcHandler::SwFilterCgcHandler(VpInterface& vpInterface) :
+    SwFilterFeatureHandler(vpInterface, FeatureTypeCgc),
+    m_swFilterFactory(vpInterface)
+{}
+SwFilterCgcHandler::~SwFilterCgcHandler()
+{}
+
+bool SwFilterCgcHandler::IsFeatureEnabled(VP_PIPELINE_PARAMS& params, bool isInputSurf, int surfIndex, SwFilterPipeType pipeType)
+{
+    VP_FUNC_CALL();
+
+    if (!SwFilterFeatureHandler::IsFeatureEnabled(params, isInputSurf, surfIndex, pipeType))
+    {
+        return false;
+    }
+
+    // BT2020YUV->BT601/709YUV enable GC in Vebox for BT2020 to RGB process
+    // SFC for other format conversion. For such case, add Cgc only to input
+    // pipe for 1 to 1 or N to 1 case.
+    if (isInputSurf && (SwFilterPipeType1ToN == pipeType) ||
+        !isInputSurf && (SwFilterPipeType1To1 == pipeType || SwFilterPipeTypeNTo1 == pipeType))
+    {
+        return false;
+    }
+
+    PVPHAL_SURFACE inputSurf  = static_cast<PVPHAL_SURFACE>(isInputSurf ? params.pSrc[surfIndex] : params.pSrc[0]);
+    PVPHAL_SURFACE outputSurf = static_cast<PVPHAL_SURFACE>(isInputSurf ? params.pTarget[0] : params.pTarget[surfIndex]);
+
+    if (inputSurf && outputSurf &&
+        IS_COLOR_SPACE_BT2020_YUV(inputSurf->ColorSpace) &&
+      (!(inputSurf->pHDRParams && 
+        (inputSurf->pHDRParams->EOTF != VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR) &&
+       !(outputSurf->pHDRParams && 
+        (outputSurf->pHDRParams->EOTF != VPHAL_HDR_EOTF_TRADITIONAL_GAMMA_SDR))))) // When HDR Enabled, GC should always be turn off, not to create the sw filter
+    {
+        if ((outputSurf->ColorSpace == CSpace_BT601) ||
+            (outputSurf->ColorSpace == CSpace_BT709) ||
+            (outputSurf->ColorSpace == CSpace_BT601_FullRange) ||
+            (outputSurf->ColorSpace == CSpace_BT709_FullRange) ||
+            (outputSurf->ColorSpace == CSpace_stRGB) ||
+            (outputSurf->ColorSpace == CSpace_sRGB))
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+SwFilter* SwFilterCgcHandler::CreateSwFilter()
+{
+    VP_FUNC_CALL();
+
+    SwFilter* swFilter = nullptr;
+    swFilter = m_swFilterFactory.Create();
+
+    if (swFilter)
+    {
+        swFilter->SetFeatureType(FeatureTypeCgc);
+    }
+
+    return swFilter;
+}
+
+void SwFilterCgcHandler::Destory(SwFilter*& swFilter)
+{
+    VP_FUNC_CALL();
+
+    SwFilterCgc* filter = nullptr;
+    filter = dynamic_cast<SwFilterCgc*>(swFilter);
+    m_swFilterFactory.Destory(filter);
+    return;
+}
