@@ -30,9 +30,8 @@
 #include "vp_utils.h"
 #include "hw_filter.h"
 #include "sw_filter_pipe.h"
-#ifndef ENABLE_VP_SOFTLET_BUILD
-#include "vp_vebox_cmd_packet_legacy.h"
-#endif
+#include "vp_platform_interface.h"
+#include "vp_hal_ddi_utils.h"
 
 using namespace vp;
 
@@ -74,13 +73,14 @@ MOS_STATUS VpScalingFilter::SfcAdjustBoundary(
     VP_FUNC_CALL();
 
     VP_PUBLIC_CHK_NULL_RETURN(m_pvpMhwInterface);
-    VP_PUBLIC_CHK_NULL_RETURN(m_pvpMhwInterface->m_sfcInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(m_pvpMhwInterface->m_vpPlatformInterface);
     VP_PUBLIC_CHK_NULL_RETURN(pdwSurfaceWidth);
     VP_PUBLIC_CHK_NULL_RETURN(pdwSurfaceHeight);
 
-    uint32_t widthAlignUnit     = m_pvpMhwInterface->m_sfcInterface->m_veWidthAlignment;
-    uint32_t heightAlignUnit    = m_pvpMhwInterface->m_sfcInterface->m_veHeightAlignment;
-    VP_PUBLIC_CHK_STATUS_RETURN(m_pvpMhwInterface->m_sfcInterface->GetInputFrameWidthHeightAlignUnit(widthAlignUnit, heightAlignUnit,
+    uint32_t widthAlignUnit  = 0;
+    uint32_t heightAlignUnit = 0;
+
+    VP_PUBLIC_CHK_STATUS_RETURN(m_pvpMhwInterface->m_vpPlatformInterface->GetInputFrameWidthHeightAlignUnit(m_pvpMhwInterface, widthAlignUnit, heightAlignUnit,
         m_bVdbox, m_codecStandard, m_jpegChromaType));
 
     uint32_t dwVeboxHeight = m_scalingParams.input.dwHeight;
@@ -124,18 +124,23 @@ void VpScalingFilter::GetFormatWidthHeightAlignUnit(
     bool bOutput,
     bool bRotateNeeded,
     uint16_t & widthAlignUnit,
-    uint16_t & heightAlignUnit)
+    uint16_t & heightAlignUnit,
+    bool isInterlacedScaling)
 {
     VP_FUNC_CALL();
 
     widthAlignUnit = 1;
     heightAlignUnit = 1;
 
-    switch (VpUtils::GetSurfaceColorPack(format))
+    switch (VpHalDDIUtils::GetSurfaceColorPack(format))
     {
     case VPHAL_COLORPACK_420:
         widthAlignUnit = 2;
         heightAlignUnit = 2;
+        if (isInterlacedScaling && bOutput)
+        {
+            heightAlignUnit = 4;
+        }
         break;
     case VPHAL_COLORPACK_422:
         widthAlignUnit = 2;
@@ -191,9 +196,9 @@ MOS_STATUS VpScalingFilter::SetColorFillParams()
         {
             VP_PUBLIC_NORMALMESSAGE("colorFillColorDst need be recalculated.");
             // Clean history Dst BG Color if hit unsupported format
-            if (!VpUtils::GetCscMatrixForRender8Bit(&m_colorFillColorDst, &Src, src_cspace, dst_cspace))
+            if (!VpHalDDIUtils::GetCscMatrixForRender8Bit(&m_colorFillColorDst, &Src, src_cspace, dst_cspace))
             {
-                VP_PUBLIC_NORMALMESSAGE("VpHal_CSC_8 failed!");
+                VP_PUBLIC_NORMALMESSAGE("VpHalDDIUtils::GetCscMatrixForRender8Bit failed!");
                 MOS_ZeroMemory(&m_colorFillColorDst, sizeof(m_colorFillColorDst));
             }
             // store the values for next iteration
@@ -315,8 +320,8 @@ MOS_STATUS VpScalingFilter::SetRectSurfaceAlignment(bool isOutputSurf, uint32_t
     MOS_STATUS eStatus = MOS_STATUS_SUCCESS;
 
     GetFormatWidthHeightAlignUnit(isOutputSurf ? m_scalingParams.formatOutput : m_scalingParams.formatInput,
-        isOutputSurf, m_scalingParams.rotation.rotationNeeded, wWidthAlignUnit, wHeightAlignUnit);
-    GetFormatWidthHeightAlignUnit(m_scalingParams.formatOutput, true, m_scalingParams.rotation.rotationNeeded, wWidthAlignUnitForDstRect, wHeightAlignUnitForDstRect);
+        isOutputSurf, m_scalingParams.rotation.rotationNeeded, wWidthAlignUnit, wHeightAlignUnit, false);
+    GetFormatWidthHeightAlignUnit(m_scalingParams.formatOutput, true, m_scalingParams.rotation.rotationNeeded, wWidthAlignUnitForDstRect, wHeightAlignUnitForDstRect, false);
 
     // The source rectangle is floored to the aligned unit to
     // get rid of invalid data(ex: an odd numbered src rectangle with NV12 format
@@ -362,7 +367,82 @@ MOS_STATUS VpScalingFilter::SetRectSurfaceAlignment(bool isOutputSurf, uint32_t
     return eStatus;
 }
 
-// Prepare
+MOS_STATUS VpScalingFilter::SetTargetRectangle(uint16_t iWidthAlignUnit, uint16_t iHeightAlignUnit, uint16_t oWidthAlignUnit, uint16_t oHeightAlignUnit, float scaleX, float scaleY)
+{
+    VP_FUNC_CALL();
+    VP_PUBLIC_CHK_NULL_RETURN(m_pvpMhwInterface);
+    if (MEDIA_IS_SKU(m_pvpMhwInterface->m_skuTable, FtrSFCTargetRectangle))
+    {
+        m_sfcScalingParams->bRectangleEnabled = m_scalingParams.bTargetRectangle;
+        uint32_t dstTargetLeftAligned  = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.output.rcDst.left, oWidthAlignUnit);
+        uint32_t dstTargetTopAligned   = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.output.rcDst.top, oHeightAlignUnit);
+        uint32_t dstTargetRightAligned = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.output.rcDst.right, oWidthAlignUnit);
+        uint32_t dstTargetDownAligned  = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.output.rcDst.bottom, oHeightAlignUnit);
+
+        // Source rectangle is not contained with target rectangle
+        uint32_t top_shift    = MOS_MAX(m_scalingParams.output.rcDst.top, m_scalingParams.input.rcDst.top) - m_scalingParams.input.rcDst.top;
+        uint32_t left_shift   = MOS_MAX(m_scalingParams.output.rcDst.left, m_scalingParams.input.rcDst.left) - m_scalingParams.input.rcDst.left;
+        uint32_t bottom_shift = m_scalingParams.input.rcDst.bottom - MOS_MIN(m_scalingParams.output.rcDst.bottom, m_scalingParams.input.rcDst.bottom);
+        uint32_t right_shift  = m_scalingParams.input.rcDst.right - MOS_MIN(m_scalingParams.output.rcDst.right, m_scalingParams.input.rcDst.right);
+
+        uint32_t srcTop = 0, srcLeft = 0, srcBottom = 0, srcRight = 0;
+        uint32_t dstTop = 0, dstLeft = 0, dstBottom = 0, dstRight = 0;
+        uint32_t dstInputLeftAligned = 0, dstInputTopAligned = 0;
+
+        if (top_shift > 0 || left_shift > 0 || bottom_shift > 0 || right_shift > 0)
+        {
+            srcTop                                  = m_scalingParams.input.rcSrc.top + MOS_UF_ROUND(top_shift / scaleY);
+            srcLeft                                 = m_scalingParams.input.rcSrc.left + MOS_UF_ROUND(left_shift / scaleX);
+            srcBottom                               = m_scalingParams.input.rcSrc.bottom - MOS_UF_ROUND(bottom_shift / scaleY);
+            srcRight                                = m_scalingParams.input.rcSrc.right - MOS_UF_ROUND(right_shift / scaleX);
+            m_sfcScalingParams->dwSourceRegionWidth  = MOS_ALIGN_FLOOR(srcRight - srcLeft, iWidthAlignUnit);
+            m_sfcScalingParams->dwSourceRegionHeight = MOS_ALIGN_FLOOR(srcBottom - srcTop, iHeightAlignUnit);
+            m_sfcScalingParams->dwSourceRegionHorizontalOffset = MOS_ALIGN_FLOOR(srcLeft, iWidthAlignUnit);
+            m_sfcScalingParams->dwSourceRegionVerticalOffset   = MOS_ALIGN_FLOOR(srcTop, iHeightAlignUnit);
+
+            dstTop = MOS_MAX(m_scalingParams.output.rcDst.top, m_scalingParams.input.rcDst.top);
+            dstLeft = MOS_MAX(m_scalingParams.output.rcDst.left, m_scalingParams.input.rcDst.left);
+            dstBottom = MOS_MIN(m_scalingParams.output.rcDst.bottom, m_scalingParams.input.rcDst.bottom);
+            dstRight  = MOS_MIN(m_scalingParams.output.rcDst.right, m_scalingParams.input.rcDst.right);
+
+            m_sfcScalingParams->dwScaledRegionWidth  = MOS_ALIGN_FLOOR(dstRight - dstLeft, oWidthAlignUnit);
+            m_sfcScalingParams->dwScaledRegionHeight = MOS_ALIGN_FLOOR(dstBottom - dstTop, oHeightAlignUnit);
+
+            dstInputLeftAligned = MOS_ALIGN_FLOOR(dstLeft, oWidthAlignUnit);
+            dstInputTopAligned  = MOS_ALIGN_FLOOR(dstTop, oHeightAlignUnit);
+        }
+        else
+        {
+            dstInputLeftAligned = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.input.rcDst.left, oWidthAlignUnit);
+            dstInputTopAligned  = MOS_ALIGN_FLOOR((uint32_t)m_scalingParams.input.rcDst.top, oHeightAlignUnit);
+        }
+
+
+        if (m_scalingParams.rotation.rotationNeeded)
+        {
+            m_sfcScalingParams->dwScaledRegionHorizontalOffset         = dstInputTopAligned;
+            m_sfcScalingParams->dwScaledRegionVerticalOffset           = dstInputLeftAligned;
+
+            m_sfcScalingParams->dwTargetRectangleStartHorizontalOffset = dstTargetTopAligned;
+            m_sfcScalingParams->dwTargetRectangleStartVerticalOffset   = dstTargetLeftAligned;
+            m_sfcScalingParams->dwTargetRectangleEndHorizontalOffset   = dstTargetDownAligned;
+            m_sfcScalingParams->dwTargetRectangleEndVerticalOffset     = dstTargetRightAligned;
+        }
+        else
+        {
+            m_sfcScalingParams->dwScaledRegionHorizontalOffset         = dstInputLeftAligned;
+            m_sfcScalingParams->dwScaledRegionVerticalOffset           = dstInputTopAligned;
+
+            m_sfcScalingParams->dwTargetRectangleStartHorizontalOffset = dstTargetLeftAligned;
+            m_sfcScalingParams->dwTargetRectangleStartVerticalOffset   = dstTargetTopAligned;
+            m_sfcScalingParams->dwTargetRectangleEndHorizontalOffset   = dstTargetRightAligned;
+            m_sfcScalingParams->dwTargetRectangleEndVerticalOffset     = dstTargetDownAligned;
+        }
+
+    }
+    return MOS_STATUS_SUCCESS;
+}
+    // Prepare
 MOS_STATUS VpScalingFilter::SetExecuteEngineCaps(
     FeatureParamScaling     &scalingParams,
     VP_EXECUTE_CAPS         vpExecuteCaps)
@@ -441,7 +521,8 @@ MOS_STATUS VpScalingFilter::CalculateEngineParams()
             true,
             m_scalingParams.rotation.rotationNeeded,
             wOutputWidthAlignUnit,
-            wOutputHeightAlignUnit);
+            wOutputHeightAlignUnit,
+            m_scalingParams.interlacedScalingType == ISCALING_INTERLEAVED_TO_INTERLEAVED);
 
         // Apply alignment restriction to Region of the input frame.
         GetFormatWidthHeightAlignUnit(
@@ -449,7 +530,8 @@ MOS_STATUS VpScalingFilter::CalculateEngineParams()
             false,
             m_scalingParams.rotation.rotationNeeded,
             wInputWidthAlignUnit,
-            wInputHeightAlignUnit);
+            wInputHeightAlignUnit,
+            m_scalingParams.interlacedScalingType == ISCALING_INTERLEAVED_TO_INTERLEAVED);
 
         m_sfcScalingParams->dwOutputFrameHeight = MOS_ALIGN_CEIL(m_scalingParams.output.dwHeight, wOutputHeightAlignUnit);
         m_sfcScalingParams->dwOutputFrameWidth  = MOS_ALIGN_CEIL(m_scalingParams.output.dwWidth, wOutputWidthAlignUnit);
@@ -514,6 +596,11 @@ MOS_STATUS VpScalingFilter::CalculateEngineParams()
             m_sfcScalingParams->dwScaledRegionVerticalOffset   = dstInputTopAligned;
         }
 
+        if (m_scalingParams.bTargetRectangle)
+        {
+            VP_RENDER_CHK_STATUS_RETURN(SetTargetRectangle(wInputWidthAlignUnit, wOutputHeightAlignUnit, wOutputWidthAlignUnit, wOutputHeightAlignUnit, fScaleX, fScaleY));
+        }
+
         // Refine the Scaling ratios in the X and Y direction. SFC output Scaled size may be changed based on the restriction of SFC alignment.
         // The scaling ratio could be changed and not equal to the fScaleX/Y.
         // Driver must make sure that the scaling ratio should be matched with the output/input size before send to HW
@@ -643,18 +730,11 @@ bool VpSfcScalingParameter::SetPacketParam(VpCmdPacket *pPacket)
         return false;
     }
 
-    VpVeboxCmdPacket *packet = dynamic_cast<VpVeboxCmdPacket *>(pPacket);
+    VpVeboxCmdPacketBase *packet = dynamic_cast<VpVeboxCmdPacketBase *>(pPacket);
     if (packet)
     {
         return MOS_SUCCEEDED(packet->SetScalingParams(params));
     }
-#ifndef ENABLE_VP_SOFTLET_BUILD
-    VpVeboxCmdPacketLegacy *packetLegacy = dynamic_cast<VpVeboxCmdPacketLegacy *>(pPacket);
-    if (packetLegacy)
-    {
-        return MOS_SUCCEEDED(packetLegacy->SetScalingParams(params));
-    }
-#endif
 
     VP_PUBLIC_ASSERTMESSAGE("Invalid packet for sfc scaling");
     return false;
@@ -834,6 +914,7 @@ MOS_STATUS PolicySfcScalingHandler::UpdateFeaturePipe(VP_EXECUTE_CAPS caps, SwFi
         {
             SwFilterScaling *filter2ndPass = featureScaling;
             SwFilterScaling *filter1ndPass = (SwFilterScaling *)feature.Clone();
+            VP_PUBLIC_CHK_NULL_RETURN(filter1ndPass);
 
             filter1ndPass->GetFilterEngineCaps().value = 0;
             filter1ndPass->SetFeatureType(FeatureType::FeatureTypeScaling);
@@ -892,10 +973,8 @@ MOS_STATUS PolicySfcScalingHandler::UpdateUnusedFeature(VP_EXECUTE_CAPS caps, Sw
         0 == caps.bOutputPipeFeatureInuse &&
         !feature.GetFilterEngineCaps().bEnabled)
     {
-        // To avoid scaling filter being destroyed in Policy::UpdateFeaturePipe.
-        feature.GetFilterEngineCaps().value = 0;
-        feature.GetFilterEngineCaps().forceEnableForSfc = 1;
-        feature.GetFilterEngineCaps().forceEnableForRender = 1;
+        // feature.GetFilterEngineCaps().usedForNextPass should be used here to
+        // avoid scaling filter being destroyed in Policy::UpdateFeaturePipe.
         feature.GetFilterEngineCaps().usedForNextPass = 1;
     }
     return MOS_STATUS_SUCCESS;
