@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018-2021, Intel Corporation
+* Copyright (c) 2018-2022, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -22,11 +22,10 @@
 #ifndef __VP_UTILS_H__
 #define __VP_UTILS_H__
 
-#include <mutex>
 #include "mos_util_debug.h"
-#include "mos_os.h"
-#include "vphal_common.h"
-#include "media_class_trace.h"
+#include "vp_common.h"
+
+using MosFormatArray = std::vector<MOS_FORMAT>;
 
 #define VP_UNUSED(param) (void)(param)
 //------------------------------------------------------------------------------
@@ -131,6 +130,9 @@
 #define VP_DEBUG_CHK_STATUS(_stmt)                                                \
     MOS_CHK_STATUS(MOS_COMPONENT_VP, MOS_VP_SUBCOMP_DEBUG, _stmt)
 
+#define VP_DEBUG_CHK_STATUS_RETURN(_stmt)                                         \
+    MOS_CHK_STATUS_RETURN(MOS_COMPONENT_VP, MOS_VP_SUBCOMP_DEBUG, _stmt)
+
 #define VP_DEBUG_CHK_NULL(_ptr)                                                   \
     MOS_CHK_NULL(MOS_COMPONENT_VP, MOS_VP_SUBCOMP_DEBUG, _ptr)
 
@@ -260,12 +262,143 @@ protected:
 #define VP_FUNC_CALL()
 #endif
 
+#define __VPHAL_VEBOX_OUTPUTPIPE_MODE                                   "VPOutputPipe Mode"
+#define __VPHAL_VEBOX_FEATURE_INUSE                                     "VeBox Feature In use"
+#define __VPHAL_VEBOX_DISABLE_SFC                                       "Disable SFC"
+#define __MEDIA_USER_FEATURE_VALUE_SFC_OUTPUT_DTR_DISABLE               "Disable SFC DTR"
+#define __MEDIA_USER_FEATURE_VALUE_SFC_OUTPUT_CENTERING_DISABLE         "SFC Output Centering Disable"
+#define __VPHAL_BYPASS_COMPOSITION                                      "Bypass Composition"
+#define __MEDIA_USER_FEATURE_VALUE_VEBOX_TGNE_ENABLE_VP                 "Enable Vebox GNE"
+
+#define __VPHAL_RNDR_SSD_CONTROL                                        "SSD Control"
+#define __MEDIA_USER_FEATURE_VALUE_CSC_COEFF_PATCH_MODE_DISABLE         "CSC Patch Mode Disable"
+#define __MEDIA_USER_FEATURE_VALUE_DISABLE_DN                           "Disable Dn"
+#define __MEDIA_USER_FEATURE_VALUE_DISABLE_PACKET_REUSE                 "Disable PacketReuse"
+#define __MEDIA_USER_FEATURE_VALUE_ENABLE_PACKET_REUSE_TEAMS_ALWAYS     "Enable PacketReuse Teams mode Always"
+
+#define __VPHAL_HDR_LUT_MODE                                            "HDR Lut Mode"
+#define __VPHAL_HDR_GPU_GENERTATE_3DLUT                                 "HDR GPU generate 3DLUT"
+#define __VPHAL_HDR_DISABLE_AUTO_MODE                                   "Disable HDR Auto Mode"
+#define __VPHAL_HDR_SPLIT_FRAME_PORTIONS                                "VPHAL HDR Split Frame Portions"
+#define __MEDIA_USER_FEATURE_VALUE_VPP_APOGEIOS_ENABLE                  "VP Apogeios Enabled"
+#define __VPHAL_PRIMARY_MMC_COMPRESSMODE                                "VP Primary Surface Compress Mode"
+#define __VPHAL_RT_MMC_COMPRESSMODE                                     "VP RT Compress Mode"
+
+#if (_DEBUG || _RELEASE_INTERNAL)
+#define __VPHAL_ENABLE_COMPUTE_CONTEXT                                  "VP Enable Compute Context"
+#define __VPHAL_RNDR_SCOREBOARD_CONTROL                                 "SCOREBOARD Control"
+#define __VPHAL_RNDR_CMFC_CONTROL                                       "CMFC Control"
+#define __VPHAL_ENABLE_1K_1DLUT                                         "Enable 1K 1DLUT"
+#define __VPHAL_VEBOX_HDR_MODE                                          "VeboxHDRMode"
+#define __VPHAL_HDR_ENABLE_QUALITY_TUNING                               "VPHAL HDR Enable Quality Tuning"
+#define __VPHAL_HDR_ENABLE_KERNEL_DUMP                                  "VPHAL HDR Enable Kernel Dump"
+#define __VPHAL_HDR_H2S_RGB_TM                                          "VPHAL H2S TM RGB Based"
+
+// Compression
+#define __VPHAL_MMC_ENABLE                                              "VP MMC In Use"
+#define __VPHAL_RT_MMC_COMPRESSIBLE                                     "VP RT Compressible"
+#define __VPHAL_PRIMARY_MMC_COMPRESSIBLE                                "VP Primary Surface Compressible"
+
+#define __VPHAL_RNDR_FORCE_VP_DECOMPRESSED_OUTPUT                       "FORCE VP DECOMPRESSED OUTPUT"
+#define __VPHAL_COMP_8TAP_ADAPTIVE_ENABLE                               "8-TAP Enable"
+#define __VPHAL_VEBOX_FORCE_VP_MEMCOPY_OUTPUTCOMPRESSED                 "Force VP Memorycopy Outputcompressed"
+#define __VPHAL_ENABLE_SFC_NV12_P010_LINEAR_OUTPUT                      "Enable SFC NV12 P010 Linear Output"
+#define __VPHAL_ENABLE_SFC_RGBP_RGB24_OUTPUT                            "Enable SFC RGBP RGB24 Output"
+
+#define __VPHAL_DBG_PARAM_DUMP_OUTFILE_KEY_NAME                         "outxmlLocation"
+#define __VPHAL_DBG_PARAM_DUMP_START_FRAME_KEY_NAME                     "startxmlFrame"
+#define __VPHAL_DBG_PARAM_DUMP_END_FRAME_KEY_NAME                       "endxmlFrame"
+#define __VPHAL_DBG_DUMP_OUTPUT_DIRECTORY                               "Vphal Debug Dump Output Directory"
+#define __VPHAL_DBG_PARA_DUMP_ENABLE_SKUWA_DUMP                         "enableSkuWaDump"
+
+#define __MEDIA_USER_FEATURE_VALUE_INTER_FRAME_MEMORY_NINJA_START_COUNTER "InterFrameNinjaStartCounter"
+#define __MEDIA_USER_FEATURE_VALUE_INTER_FRAME_MEMORY_NINJA_END_COUNTER "InterFrameNinjaEndCounter"
+#define __MEDIA_USER_FEATURE_VALUE_ENABLE_IFNCC                         "EnableIFNCC"
+#endif  //(_DEBUG || _RELEASE_INTERNAL)
 
 class VpUtils
 {
 public:
-    // it is only be used by vpdata->pVpHalState->CopySurface, will be removed after mediaCopy ready
-    static MOS_SURFACE VpHalConvertVphalSurfaceToMosSurface(PVPHAL_SURFACE surface);
+    //!
+    //! \brief    Allocates the Surface
+    //! \details  Allocates the Surface
+    //!           - if the surface is not already allocated OR
+    //!           - resource dimenisions OR format changed
+    //! \param    [in] pOsInterface
+    //!           Pointer to MOS_INTERFACE
+    //! \param    [in,out] pSurface
+    //!           Pointer to VPHAL_SURFACE
+    //! \param    [in] pSurfaceName
+    //!           Pointer to surface name
+    //! \param    [in] format
+    //!           Expected MOS_FORMAT
+    //! \param    [in] DefaultResType
+    //!           Expected Resource Type
+    //! \param    [in] DefaultTileType
+    //!           Expected Surface Tile Type
+    //! \param    [in] dwWidth
+    //!           Expected Surface Width
+    //! \param    [in] dwHeight
+    //!           Expected Surface Height
+    //! \param    [in] bCompressible
+    //!           Surface being compressible or not
+    //! \param    [in] CompressionMode
+    //!           Compression Mode
+    //! \param    [out] pbAllocated
+    //!           true if allocated, false for not
+    //! \param    [in] resUsageType
+    //!           resource usage type for caching
+    //! \param    [in] tileModeByForce
+    //!           Forced tile mode
+    //! \param    [in] memType
+    //!           vidoe memory location
+    //! \param    [in] isNotLockable
+    //!           Flag to indicate whether resource being not lockable
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success. Error code otherwise
+    //!
+    static MOS_STATUS ReAllocateSurface(
+        PMOS_INTERFACE        osInterface,                               
+        PVPHAL_SURFACE        surface,                                   
+        PCCHAR                surfaceName,                               
+        MOS_FORMAT            format,                                     
+        MOS_GFXRES_TYPE       defaultResType,                            
+        MOS_TILE_TYPE         defaultTileType,                            
+        uint32_t              dwWidth,                                    
+        uint32_t              dwHeight,                                   
+        bool                  bCompressible,                             
+        MOS_RESOURCE_MMC_MODE compressionMode,                            
+        bool                  *bAllocated,                                
+        MOS_HW_RESOURCE_DEF   resUsageType    = MOS_HW_RESOURCE_DEF_MAX,  
+        MOS_TILE_MODE_GMM     tileModeByForce = MOS_TILE_UNSET_GMM,       
+        Mos_MemPool           memType         = MOS_MEMPOOL_VIDEOMEMORY,  
+        bool                  isNotLockable   = false);                                         
+
+    //!
+    //! \brief
+    //! \details  Get CSC matrix in a form usable by Vebox, SFC and IECP kernels
+    //! \param    [in] SrcCspace
+    //!           Source Cspace
+    //! \param    [in] DstCspace
+    //!           Destination Cspace
+    //! \param    [out] pfCscCoeff
+    //!           [3x3] Coefficients matrix
+    //! \param    [out] pfCscInOffset
+    //!           [3x1] Input Offset matrix
+    //! \param    [out] pfCscOutOffset
+    //!           [3x1] Output Offset matrix
+    //! \return   void
+    //!
+    static void GetCscMatrixForVeSfc8Bit(
+        VPHAL_CSPACE srcCspace,
+        VPHAL_CSPACE dstCspace,
+        float        *fCscCoeff,
+        float        *fCscInOffset,
+        float        *fCscOutOffset);
+
+    static bool IsSyncFreeNeededForMMCSurface(PVPHAL_SURFACE surface, PMOS_INTERFACE osInterface);
+
+MEDIA_CLASS_DEFINE_END(VpUtils)
 };
 
 #endif // !__VP_UTILS_H__
